/*
	Lip Tracking | Shilpi Nayak and Vanna Bushong | CS 269, UCLA
	http://www.vannabushong.com/Vision/CS269.html

	Credit for the dynamic programming snake goes to Garett Ridge (UCLA), whose
	code we modifed and reimplemented for lip tracking purposes. The original 
	version can be found here: http://cs.ucla.edu/~garett/dp_snake_code.html
*/

#include <opencv/cv.h>
#include <opencv/highgui.h>
#include <opencv/cxcore.h>

using namespace std;
using namespace cv;

#include <iostream>
#include <cmath>
#include <string>  
#include <fstream>
#include <sstream>

IplImage *colorFrame, *grayFrame, *gradientImage;	// Original frames - color, grayscale, and gradient
IplImage *cannyFrame;								// Canny frame
IplImage *rgbFrame, *gradRGB;						// Red hue extracted, Gradient of hue

// Variables for canny edge detection
int edgeThresh = 1;
int lowThreshold = 16;
int const max_lowThreshold = 100;
int ratio = 3;
int kernel_size = 3;
char* window_name = "Edge Map";
int run=0;

//td::vector<cv::Point> pvect;

// For face detection
CvHaarClassifierCascade *cascade;
CvMemStorage *storage;

// Snake constants
const int MAX_SNAXELS = 10000, NUM_NEIGHBORS = 9;
const float alpha = 100, beta = 250, w_line = 250, w_edge = 50, w_canny = 100, w_hue = 500, min_dist_constraint = 3;
const int initialSmooth = 51, initialIterationsPerDeblurStep = 30, iterationsPerDeblurStep_delta = 30, 
/*must be even*/ smoothFactor_delta = 16;
float MAX = 10000000;

class DPSnake;
DPSnake *s;

// Defines a snaxel
class snakePoint 
{
	public: int x,y; 

    snakePoint( int x, int y) {
		this->x=x;
		this->y=y;
	} 
	   
	snakePoint() {}

    snakePoint operator+ (const snakePoint& p) const { return snakePoint(this->x + p.x,this->y + p.y); }
};

// Initial snaxels based on coordinates of the mouth box. prevBottomLeft is used
// for calculating the shift of the mouth from frame to frame
snakePoint prevBottomLeft, bottomLeft = snakePoint(0,0), topRight;
int boxWidth, boxHeight;

// The main class
class DPSnake {

public:       

	int snaxelsAdded; 
	int i, j, k, l, pos_j, pos_k, min_position_k, min_position_l, min_final_position_j, min_final_position_k;
	float min_energy, energy, min_final_energy, min_final_energy_prev;
	snakePoint nextNode , currNode, prevNode, snaxel[MAX_SNAXELS], neighbor[NUM_NEIGHBORS];
	float energyMatrix[MAX_SNAXELS-1][NUM_NEIGHBORS][NUM_NEIGHBORS];
	int positionMatrix[MAX_SNAXELS-1][NUM_NEIGHBORS][NUM_NEIGHBORS][2];
	IplImage *outputImage, *outputImage2;
	snakePoint minxSnake,maxxSnake,minySnake[5],maxySnake,centerEllipse;
    int ellipseA,ellipseBUp,ellipseBDown;   
	float bDownSquared,aSquared,bUpSquared;


	int getPixel (IplImage* img, int x, int y)      //this function is for 8-bit 1-channel images
	{      
		return ((uchar*)(img->imageData + img->widthStep*y))[x];      
	}


	int getPixelRGB (IplImage* img, int x, int y)      //this function is for 8-bit 1-channel images
	{      
		return ((uchar*)(img->imageData + img->widthStep*y))[x];      
	}


	void setPixel (IplImage* img, int x, int y, int value) //this function is for 8-bit 1-channel images
	{      
		((uchar*)(img->imageData + img->widthStep*y))[x] = value;     
	}


	// Draws a red snake on the output image
    void drawLinesBetweenSnaxels()
    {             
		for (int i = 0; i < snaxelsAdded-1; i++)
		{
			cvLine(outputImage2, cvPoint(snaxel[i].x, snaxel[i].y), cvPoint(snaxel[i+1].x, snaxel[i+1].y), cvScalar(0, 0, 255), 1, 8, 0);
		}	
		// Connect the last snaxel to the first, closing the loop
		cvLine(outputImage2, cvPoint(snaxel[snaxelsAdded-1].x, snaxel[snaxelsAdded-1].y), cvPoint(snaxel[0].x, snaxel[0].y), cvScalar(0, 0, 255), 1, 8, 0);
    }


	// Draws an ellipse at the center of the mouth. The goal was to give pixels in the ellipse 
	// the maximum energy value so the snake would not collapse.
	void drawEllipse()	
    {             
		
		int hoffx = (maxxSnake.x+minxSnake.x);
		int hoffy = (maxxSnake.y+minxSnake.y);
		int xcenter=0, ycenter=0;		
		for(int i=0;i<5;i++)
		{
			xcenter=xcenter + minySnake[i].x;
		    ycenter+=minySnake[i].y;
	    }
		xcenter=xcenter/5;
		ycenter=ycenter/5;
		int voffy = (maxySnake.y+ycenter);
	    cvCircle(outputImage2,cvPoint(xcenter, ycenter + voffy/2),2,cvScalar(0,0,0),1,8,0);
	    cvCircle(outputImage2,cvPoint(hoffx/2,hoffy/2),2,cvScalar(255,255,255),1,8,0);
     	centerEllipse.x = (xcenter + hoffx/2)/2;
		
		centerEllipse.y = (hoffy/2);
		//RotatedRect box = fitEllipse(Mat(pvect));
		ellipseA=maxxSnake.x-hoffx/2-10;
		ellipseBDown=(maxySnake.y-centerEllipse.y)-5;
		ellipseBUp=centerEllipse.y-ycenter-5;
        cvEllipse (outputImage2,cvPoint(centerEllipse.x,centerEllipse.y),cvSize(ellipseA,ellipseBDown),0,180,0,cvScalar(0,255,255),1,8,0);
        cvEllipse (outputImage2,cvPoint(centerEllipse.x,centerEllipse.y),cvSize(ellipseA,ellipseBUp),0,0,-180,cvScalar(0,255,255),1,8,0);
		//CvPoint center; CvSize size;
        //center = cvPointFrom32f(box.center);
        //size.width = cvRound(box.size.width*0.5);
		//size.height = cvRound(box.size.height*0.5);	 
		// cvEllipse(outputImage2, center, size,-box.angle, 0, 360,CV_RGB(0,0,255), 1, CV_AA, 0);
	} 


    // Overwrite *gradient with an image showing the gradient / edge map of the image in src.
    void obtainGradient(IplImage* src, IplImage *gradient)
    {
		IplImage *grad_x = cvCreateImage(cvSize(src->width,src->height),IPL_DEPTH_8U,1);			  
		IplImage *grad_y = cvCreateImage(cvSize(src->width,src->height),IPL_DEPTH_8U,1);
		IplImage *grad_dx = cvCreateImage(cvSize(src->width,src->height),IPL_DEPTH_16S,1);
		IplImage *grad_dy = cvCreateImage(cvSize(src->width,src->height),IPL_DEPTH_16S,1);

		cvSetZero(gradient);

        cvSobel(src, grad_dx, 1, 0, 3 );
		cvConvertScaleAbs(grad_dx, grad_x);      //gradient of the image in x direction
		
		cvSobel(src, grad_dy, 0, 1, 3 );
		cvConvertScaleAbs(grad_dy, grad_y);      //gradient of the image in y direction

		int pixelX, pixelY, pixel;

        for (int x = 0; x < src->width; x++)
		{
            for (int y = 0; y < src->height; y++)
            {
                pixelX = getPixel(grad_x, x, y);

                pixelY = getPixel(grad_y, x, y);

                pixel = int(sqrt((float)(pixelX*pixelX+pixelY*pixelY)));

                if(pixel > 255) pixel=255;//truncate

                setPixel(gradient,x,y,pixel);
            } 
		}
 
        //release used images;
        cvReleaseImage(&grad_x);
        cvReleaseImage(&grad_y);
        cvReleaseImage(&grad_dx);
        cvReleaseImage(&grad_dy);
	}


    // Verify that p is in the image.
    bool checkBounds(snakePoint p) 
	{
		if(p.x > grayFrame->width || p.x < 0 || p.y > grayFrame->height || p.y < 0) return false;
        return true;
    }


	//External energy term in the energy formula.
	float externalEnergy(snakePoint curr)
	{		
		if(curr.x > bottomLeft.x && curr.x < topRight.x && curr.y > topRight.y && curr.y < bottomLeft.y)
		{		
			aSquared=pow(ellipseA,2.0);
			bUpSquared=pow(ellipseBUp,2.0);
			bDownSquared=pow(ellipseBDown,2.0);
			float distUp=(pow((curr.x-centerEllipse.x),2.0)/aSquared)+(pow((curr.y-centerEllipse.y),2.0)/bUpSquared);			
			float distDown=(pow((curr.x-centerEllipse.x),2.0)/aSquared)+(pow((curr.y-centerEllipse.y),2.0)/bDownSquared);

	        if((curr.y>centerEllipse.y&&distDown<=1||curr.y<centerEllipse.y&&distUp<=1)&&run==1)
	     	{
				return MAX;          
		    }
		    else
		    {
			return (float)(w_line * getPixel(rgbFrame,curr.x,curr.y) - w_canny * pow(getPixel(cannyFrame,curr.x,curr.y), 2.0) - w_hue * pow(getPixel(gradRGB,curr.x,curr.y), 2.0));
			}
		}
		else
		{
			return MAX;
		}					
	}


	// Energy equation
	float E(snakePoint prev, snakePoint curr, snakePoint next)
	{
		int distanceSquared = (int)pow(next.x - curr.x, 2.0) + (int)pow(next.y - curr.y, 2.0); //|vk+1 - vk|^2            

		int derivative = (int)pow(next.x - 2*curr.x + prev.x, 2.0) + (int)pow(next.y - 2*curr.y +prev.y, 2.0); //|vk+1 - 2*vk + vk-1|^2

		return (float) .5 * (alpha * distanceSquared + beta * derivative + externalEnergy(curr));
	}
     

	//Energy equation (case of first snaxel - no previous snake component)
	float E(snakePoint curr, snakePoint next)
	{
		int distanceSquared = (int)pow(next.x - curr.x, 2.0) + (int)pow(next.y - curr.y, 2.0); //|vk+1 - vk|^2            

		return (float) alpha * distanceSquared + externalEnergy(curr);
	}


	void iterateSnake()
	{
		for (i=0;i<snaxelsAdded-1;i++)    // For each snaxel, find the optimum next position
		{
			//intialize the matrices to default values
			for(j=0;j<NUM_NEIGHBORS;j++) for(k=0;k<NUM_NEIGHBORS;k++) 
				{energyMatrix[i][j][k] = FLT_MAX; positionMatrix[i][j][k][0] = 0; positionMatrix[i][j][k][1] = 0;}

			for (j=0;j<NUM_NEIGHBORS;j++) // for all neighbors of next node
			{
				nextNode = snaxel[i+1] + neighbor[j];
				if(!checkBounds(nextNode)) continue;
				min_energy = FLT_MAX;
				for (k=0;k<NUM_NEIGHBORS;k++) // for all neighors of curr node
				{
					currNode = snaxel[i] + neighbor[k];
					float distance = (float)sqrt(pow(nextNode.x - currNode.x, 2.0) + pow(nextNode.y - currNode.y, 2.0));
					if(!checkBounds(currNode) || distance < min_dist_constraint) continue;

					if(i>=0)       //Previous node is defined
					{
						min_energy = FLT_MAX;

						for (l=0;l<NUM_NEIGHBORS;l++) //for all neighors of previous node

						{
							if (i==0)
							{
								prevNode = snaxel[snaxelsAdded-1] + neighbor[l];
							}
							else 
							{
								prevNode = snaxel[i-1] + neighbor[l];
							}

							if(!checkBounds(prevNode)) continue;

                            if (i==0)
							{
								energy = energyMatrix[snaxelsAdded-1][k][l] + E(prevNode, currNode, nextNode);
							}
							else 
							{
								energy = energyMatrix[i-1][k][l] + E(prevNode, currNode, nextNode);
							}

							//energy = energyMatrix[i-1][k][l] + E(prevNode, currNode, nextNode);

							if(i==snaxelsAdded-2) energy += externalEnergy(nextNode);       //handle last node

							if (energy < min_energy)
							{
								min_energy = energy;
								min_position_k = k;
								min_position_l = l;
							}
						}
					}

					else   //handle first node
					{
						energy = E(currNode, nextNode);

						if (energy < min_energy)
						{
							min_energy = energy;
							min_position_k = k;
							min_position_l = 0;
						}
					}

					// Store minimum energy into table
					energyMatrix[i][j][k] = min_energy;
					positionMatrix[i][j][k][0] = min_position_k;
					positionMatrix[i][j][k][1] = min_position_l;
				}                                 
			}
		}

		// search final column for minimum energy
		min_final_energy = FLT_MAX;
		min_final_position_j = min_final_position_k = 0;

		for (j=0; j<NUM_NEIGHBORS; j++)

			for (k=0; k<NUM_NEIGHBORS; k++)
			{
				if(energyMatrix[snaxelsAdded-2][j][k] < min_final_energy)
				{
					min_final_energy = energyMatrix[snaxelsAdded-2][j][k];
					min_final_position_j = j;
					min_final_position_k = k;
				}
			}

		pos_j = min_final_position_j;

		pos_k = min_final_position_k;     
		
		// search backwards through table to find optimum postions 
		for(i=snaxelsAdded-1; i>=0; i--)
		{
			snaxel[i] = snaxel[i] + neighbor[pos_j];
		
			if(i>0)
			{
				pos_j = positionMatrix[i-1][pos_j][pos_k][0];
				pos_k = positionMatrix[i-1][pos_j][pos_k][1];
			}
		}

	}


	// Shift the entire snake to be within the mouth box every frame
	void shiftSnake() 
	{	
		int x_shift = bottomLeft.x - prevBottomLeft.x;
		int y_shift = bottomLeft.y - prevBottomLeft.y;

		for(i=0; i<snaxelsAdded; i++)
		{
			snaxel[i].x = snaxel[i].x + x_shift;
			snaxel[i].y = snaxel[i].y + y_shift;
		}

		for(int i=0;i<5;i++)
		{
			minySnake[i].x+= x_shift;
			minySnake[i].y+= y_shift;
		}    
		minxSnake.x+=x_shift;
		minxSnake.y+=y_shift;
		maxxSnake.x+=x_shift;
		maxxSnake.y+=y_shift;
		maxySnake.x+=x_shift;
		maxySnake.y+=y_shift;
	}


	void findPoints()
	{
		int j, k;
		for (int i = 0; i <= snaxelsAdded-1; i++)
		{

			//pvect.push_back(cvPoint(snaxel[i].x,snaxel[i].y));

				
				if(snaxel[i].x<minxSnake.x)
				{
					minxSnake=snaxel[i];
				}
				if(snaxel[i].x>maxxSnake.x)
				{
					maxxSnake=snaxel[i];
				}
				if(snaxel[i].y>maxySnake.y)
				{
					maxySnake=snaxel[i];
				}	
				if(snaxel[i].y<minySnake[4].y)
                 {                   
                   if(snaxel[i].y <minySnake[0].y)
                   {
					   for(k=4;k>0;k--)
						 {
					    	minySnake[k]=minySnake[k-1];
						 }
						 minySnake[0]=snaxel[i];
                       
                    }
                    else if(snaxel[i].y < minySnake[1].y)
                    {
					   for(k=4;k>1;k--)
					     {
						   	minySnake[k]=minySnake[k-1];
					   	 }
                        minySnake[1] = snaxel[i];
                    }
                   else if(snaxel[i].y < minySnake[2].y)
                    {
						for(k=4;k>2;k--)
					     {
						   	minySnake[k]=minySnake[k-1];
					   	 }
                        minySnake[2] = snaxel[i];
                   }
                   else if(snaxel[i].y < minySnake[3].y)
                   {
					    minySnake[4]=minySnake[3];
					   	minySnake[3] = snaxel[i];
                    }
                   else if(snaxel[i].y < minySnake[4].y)
                   {
					   minySnake[4] = snaxel[i];
                   }
                }    													
		}		
		//contours.push_back(pvect);				
	}


	//Draw the snake, then run all iterations until the snake converges.
	void runSnake()
	{             	   
		cvCopy(gradientImage, outputImage);
		cvCopy(colorFrame, outputImage2);              

		drawLinesBetweenSnaxels(); // draw a white line on each output image

		cvShowImage("Lip Tracking", outputImage2);
		cvShowImage("Gradient", outputImage);

		cvWaitKey(10);

		int counter = 0;

		int smoothFactor = initialSmooth, iterationsPerDeblurStep = initialIterationsPerDeblurStep;

		while (true)
		{      
			cvCopy(gradientImage, outputImage);
			cvCopy(colorFrame, outputImage2);      

			drawLinesBetweenSnaxels();

			cvShowImage("Gradient", outputImage);
			cvShowImage("Lip Tracking", outputImage2);

			cvWaitKey(1);                                         

			iterateSnake();
		
			// Stop when the snake has converged
			if (min_final_energy == min_final_energy_prev)
			{
				if(run==0)
				{
					s->findPoints();
					s->drawEllipse();
					run=1;				
				}			
				//cvShowImage("Lip Tracking", outputImage2);
				break;
			}

			else min_final_energy_prev = min_final_energy;
            
		}             
	}


	void initSnakePoints(snakePoint bottomLeft, snakePoint topRight, int boxWidth, int boxHeight) 
	{	
		snakePoint topLeft = snakePoint(bottomLeft.x, topRight.y);
		snakePoint bottomRight = snakePoint(topRight.x, bottomLeft.y);

		for (int x=topLeft.x; x<topLeft.x+boxWidth; x=x+10)
		{
			s->snaxel[s->snaxelsAdded++] = snakePoint(x, topLeft.y);
		}

		s->snaxel[s->snaxelsAdded++] = topRight;

		for (int y=topRight.y; y<topRight.y+boxHeight; y=y+10)
		{
			s->snaxel[s->snaxelsAdded++] = snakePoint(topRight.x, y);
		}

		s->snaxel[s->snaxelsAdded++] = bottomRight;
		   
		for (int x=bottomRight.x; x>bottomRight.x-boxWidth; x=x-10)
		{
			s->snaxel[s->snaxelsAdded++] = snakePoint(x, bottomRight.y);
		}

		s->snaxel[s->snaxelsAdded++] = bottomLeft;

		for (int y=bottomLeft.y; y>bottomLeft.y-boxHeight; y=y-10)
		{
			s->snaxel[s->snaxelsAdded++] = snakePoint(bottomLeft.x, y);
		}

		s->snaxel[s->snaxelsAdded++] = topLeft;
	}

	
	void processFrame() 
	{
		obtainGradient(grayFrame, gradientImage);
		obtainGradient(rgbFrame, gradRGB);
		
		cvShowImage("Lip Tracking", colorFrame);
		cvShowImage("Gradient", gradientImage);
		cvShowImage("RGB", rgbFrame);
		cvShowImage("gradRGB", gradRGB);
    }


	~DPSnake()
    {
		// Free the resources.
		cvReleaseImage(&outputImage);
		cvReleaseImage(&outputImage2);	
    }


	DPSnake()
	{             	
		neighbor[0] = snakePoint(-1,-1);
		neighbor[1] = snakePoint(0,-1);
		neighbor[2] = snakePoint(1,-1);
		neighbor[3] = snakePoint(-1,0);
		neighbor[4] = snakePoint(0,0);
		neighbor[5] = snakePoint(1,0);
		neighbor[6] = snakePoint(-1,1);
		neighbor[7] = snakePoint(0,1);
		neighbor[8] = snakePoint(1,1);
              
		min_final_energy_prev = FLT_MAX;
        maxxSnake.x=0, maxySnake.y=0;
		minxSnake.x=MAX;			
		minySnake[0].y=MAX-400; 
		minySnake[1].y=MAX-300; 
		minySnake[2].y=MAX-200; 
		minySnake[3].y=MAX-100; 
		minySnake[4].y=MAX; 			
	    maxxSnake.x=0, maxySnake.y=0;

		grayFrame = cvCreateImage(cvSize(colorFrame->width,colorFrame->height),IPL_DEPTH_8U,1);
			
		outputImage = cvCreateImage(cvSize(colorFrame->width,colorFrame->height),IPL_DEPTH_8U,1);
		outputImage2 = cvCreateImage(cvSize(colorFrame->width,colorFrame->height),IPL_DEPTH_8U,3);

		obtainGradient(grayFrame, gradientImage);
		obtainGradient(rgbFrame, gradRGB);
	
		cvShowImage("Lip Tracking", colorFrame);
		cvShowImage("Gradient", gradientImage);
		cvShowImage("RGB", rgbFrame);
		cvShowImage("gradRGB", gradRGB);
	}
};

	
void detectFaces(IplImage *img)
{
	prevBottomLeft = bottomLeft;

    int i;

    // Detect faces 
    CvSeq *faces = cvHaarDetectObjects(img, cascade, storage, 1.1, 3, 0, cvSize(20, 20));
 
    // For each face found, draw a red box 
	int newHeight, newWidth, offset;

    for(i=0; i<(faces ? faces->total : 0); i++) 
	{
        CvRect *r = (CvRect*)cvGetSeqElem(faces, i);

		// Uncomment this to actually draw the rectangle
        //cvRectangle(img, cvPoint(r->x, r->y), cvPoint( r->x + r->width, r->y + r->height ), CV_RGB( 0, 0, 255 ), 1, 8, 0 );

		// Box around mouth
		newHeight = r->height*0.15;
		newWidth = r->width*0.40;
		offset = r->width*0.30;

		// Uncomment this to actually draw the rectangle
		//cvRectangle(img, cvPoint( r->x + offset, r->y + r->height - r->height*.15), cvPoint( r->x + offset + newWidth, r->y + r->height - newHeight - r->height*.15 ), CV_RGB( 255, 0, 0 ), 1, 8, 0 );

		bottomLeft=snakePoint(r->x + offset, r->y + r->height - r->height*.15);
		topRight=snakePoint(r->x + offset + newWidth, r->y + r->height - newHeight - r->height*.15);
		boxWidth = newWidth;
		boxHeight = newHeight;
	}	

	cvClearSeq(faces);
}


void CannyThreshold (int, void*)
{
	cvSmooth(grayFrame, cannyFrame, CV_BLUR, 7, 7);
	cvCanny(cannyFrame, cannyFrame, lowThreshold, lowThreshold*ratio, kernel_size);
	cvShowImage(window_name, cannyFrame);
}


// Gives the gradient of image RGB in the format such that each pixel is R/R+G+B 
void convertImageRGBtoHue(const IplImage *imageRGB)
{
	float fR, fG, fB, Y;
	float temp;
	const float FLOAT_TO_BYTE = 255.0f;
	const float BYTE_TO_FLOAT = 1.0f / FLOAT_TO_BYTE;
	int redComp, blueComp, greenComp;

	// Create a blank HSV image
	//IplImage *imageHue = cvCreateImage(cvGetSize(imageRGB), 8, 1);
	if (!rgbFrame || imageRGB->depth != 8 || imageRGB->nChannels != 3) {
		printf("ERROR in convertImageRGBtoHue()! Bad input image.\n");
		exit(1);
	}
	
	int h = imageRGB->height;				// Pixel height.
	int w = imageRGB->width;				// Pixel width.
	int rowSizeRGB = imageRGB->widthStep;	// Size of row in bytes, including extra padding.
	char *imRGB = imageRGB->imageData;		// Pointer to the start of the image pixels.
	int rowSizeHue = rgbFrame->widthStep;	// Size of row in bytes, including extra padding.
	char *imHue = rgbFrame->imageData;		// Pointer to the start of the image pixels.
	
	for (int y=0; y<h; y++) 
	{
		for (int x=0; x<w; x++) 
		{
			uchar *pRGB = (uchar*)(imRGB + y*rowSizeRGB + x*3);
			int bB = *(uchar*)(pRGB+0);	// Blue component
			int bG = *(uchar*)(pRGB+1);	// Green component
			int bR = *(uchar*)(pRGB+2);	// Red component
			
			// Convert from 8-bit integers to floats.
			Y = 0.299*bB + 0.587*bG + 0.114*bR;
			fR = bR/Y*100;
			fG = bG/Y*100;
			fB = bB/Y*100;

			temp = fR+fG+bB;
		
			redComp = (int)(fR);
			blueComp = (int)(fB);
			greenComp = (int)(fG);
			
			if (greenComp > 80 && greenComp < 95 && redComp > 125 && Y < 180)
			{
				uchar *pHue = (uchar*)(imHue + y*rowSizeHue + x);			
				*(pHue) = (((int)bR/(.5*bR + .5*bG))*255-Y)*200;				
			}
			else
			{
				uchar *pHue = (uchar*)(imHue + y*rowSizeHue + x);
				*(pHue) = 0;
			}
		}
	}	
}


void main()
{
	CvCapture *capture;
    int       key = 0;
    char      *filename = "haarcascade_frontalface_alt.xml";
	int pos=0;

    cascade = (CvHaarClassifierCascade*)cvLoad(filename, 0, 0, 0);
    storage = cvCreateMemStorage(0);
    capture = cvCaptureFromCAM(0);
    assert(cascade && storage && capture);

	// Create windows
	namedWindow(window_name, CV_WINDOW_AUTOSIZE);			  
	cvNamedWindow("Gradient", CV_WINDOW_AUTOSIZE);
	cvNamedWindow("Lip Tracking", CV_WINDOW_AUTOSIZE);
	cvNamedWindow("RGB", CV_WINDOW_AUTOSIZE);
	cvNamedWindow("gradRGB", CV_WINDOW_AUTOSIZE);	
   
	while(key != 'q')
	{ 
        colorFrame = cvQueryFrame(capture);	

		if(!colorFrame) break;

		if(pos==0)
		{
			rgbFrame = cvCreateImage(cvSize(colorFrame->width,colorFrame->height),IPL_DEPTH_8U,1);
			grayFrame = cvCreateImage(cvSize(colorFrame->width, colorFrame->height), IPL_DEPTH_8U, 1);
			cannyFrame = cvCreateImage(cvSize(colorFrame->width, colorFrame->height), IPL_DEPTH_8U, 1);
			gradientImage = cvCreateImage(cvSize(colorFrame->width,colorFrame->height),IPL_DEPTH_8U,1);
			gradRGB = cvCreateImage(cvSize(colorFrame->width,colorFrame->height),IPL_DEPTH_8U,1);
		}
		
		// Extract red hue
		convertImageRGBtoHue(colorFrame);

		// Convert color frame to grayscale
		cvCvtColor(colorFrame, grayFrame, CV_BGR2GRAY);	

		// Find canny edges
		CannyThreshold(0, 0);

		// Find faces and mouths
		detectFaces(colorFrame);

		if (pos==0)
		{	
			//detectFaces(colorFrame);
			s = new DPSnake();
			s->snaxelsAdded = 0;
			s->initSnakePoints(bottomLeft, topRight, boxWidth, boxHeight);     
			pos=1;
			s->runSnake();
			//s->findPoints();
			//s->drawEllipse();
		}    
		else
		{
			s->shiftSnake();
			s->processFrame();
			s->runSnake();
		}

        // Quit if user press 'q' 
        key = cvWaitKey(10);
	}

    // free memory 
	cvDestroyWindow(window_name);
	cvDestroyWindow("RGB");
	cvDestroyWindow("Lip Tracking");
	cvDestroyWindow("Gradient");
	cvDestroyWindow("gradRGB");
    cvReleaseCapture(&capture);
	cvReleaseHaarClassifierCascade(&cascade);
    cvReleaseMemStorage(&storage);
		
	cvReleaseImage(&gradientImage);
	cvReleaseImage(&colorFrame);
	cvReleaseImage(&grayFrame);
	cvReleaseImage(&cannyFrame);
	cvReleaseImage(&rgbFrame);
	cvReleaseImage(&gradRGB);
}